/* 
 * Copyright (C) 2013 Luca Zanetti. All rights reserved.
 *
 * This file may be used under the terms of the GNU General Public License
 * versions 3.0 as published by the Free Software Foundation and
 * appearing in the COPYING file included in the packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 *  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
*/

#include <math.h>

int M = 50; /* trotter approximation */
const char INPUT1[] = "input1.dat";
const char INPUT2[] = "input2.dat";
//const int T = 10;
int acc,ref;

struct state {
	int n; /* number of vertices of the graph */
	int **replicas;
};

typedef struct state *State;

typedef int **Graph;

/* read and return the adjacency matrix of a n vertex graph from file */
Graph read_graph_f(const char *filename, int n);

/* calculate the local weight of chain c of replica r (from potential energy)*/
double diagonal_weight(State state, Graph g1, Graph g2, int c, int r, double beta, double gamma);

/* calculate the local weight of chain c of replica r (from kinetic energy):
	chain c of replica r must be in the same state as chain c of replica r+1 to be a valid plaquette */
double kinetic_weight(State state, int c, int r, double gamma, double beta);

/* sample a valid plaquette from the current configuration of chain c. Returns the replica number.
 	The seed for the random number generator must be initialized */
int sample_plaquette(State state, int c);

/* modify the local configuration of the plaquette of chain c and replicas (r,r+1)
 * returns 0 if the move has been correctly executed */
int plaquette_move(State state, int c, int r);

/* print the current configuration of replica r */
void print_state_r(State state, int r);

/* print the current configuration of chain c in every replica */
void print_state_c(State state, int c);

/* print the current configuration of every replica */
void print_state_a(State state, Graph g1, Graph g2);

/* make a move according to metropolis rule:
 * (for every chain) choose u.a.r a valid plaquette r
 * compute current weight w of the plaquette
 * flip the spins of the plaquette (modifying chain c of replicas r and r+1)
 * compute new weight w'
 * accept the move with probability min{1,w'/w} */
void metropolis_move(State state, Graph g1, Graph g2, double gamma, double beta);

/* return cost of the configuration currently assumed by replica r */
int configuration_cost(State state, Graph g1, Graph g2, int r);
